@@ -25,14 +25,8 @@
 #define WLAN_STA_PENDING_POLL BIT(6) /* pending activity poll not ACKed */
 #define WLAN_STA_PREAUTH BIT(7)
 
-#define WLAN_RATE_1M BIT(0)
-#define WLAN_RATE_2M BIT(1)
-#define WLAN_RATE_5M5 BIT(2)
-#define WLAN_RATE_11M BIT(3)
-#define WLAN_RATE_COUNT 4
-
-/* Maximum size of Supported Rates info element. IEEE 802.11 has a limit of 8,
- * but some pre-standard IEEE 802.11g products use longer elements. */
+/* Maximum number of supported rates (from both Supported Rates and Extended
+ * Supported Rates IEs). */
 #define WLAN_SUPP_RATES_MAX 32
 
 
@@ -45,7 +39,7 @@ struct sta_info {
 	u16 capability;
 	u16 listen_interval; /* or beacon_int for APs */
 	u8 supported_rates[WLAN_SUPP_RATES_MAX];
-	u8 tx_supp_rates;
+	int supported_rates_len;
 
 	enum {
 		STA_NULLFUNC = 0, STA_DISASSOC, STA_DEAUTH, STA_REMOVE
@@ -75,6 +69,7 @@ struct sta_info {
 	struct rsn_preauth_interface *preauth_iface;
 
 	struct hostapd_ssid *ssid; /* SSID selection based on (Re)AssocReq */
+	struct hostapd_ssid *ssid_probe; /* SSID selection based on ProbeReq */
 
 	int vlan_id;
 };
