@@ -1,6 +1,6 @@
 /*
  * hostapd / Configuration file
- * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -13,18 +13,20 @@
  */
 
 #include "includes.h"
+#ifndef CONFIG_NATIVE_WINDOWS
 #include <grp.h>
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 #include "hostapd.h"
 #include "driver.h"
 #include "sha1.h"
 #include "eap.h"
 #include "radius_client.h"
+#include "wpa_common.h"
 
 
 static void hostapd_config_defaults_bss(struct hostapd_bss_config *bss)
 {
-	memset(bss, 0, sizeof(*bss) + sizeof(struct hostapd_radius_servers));
 	bss->radius = (struct hostapd_radius_servers *) (bss + 1);
 
 	bss->logger_syslog_level = HOSTAPD_LEVEL_INFO;
@@ -45,6 +47,7 @@ static void hostapd_config_defaults_bss(struct hostapd_bss_config *bss)
 
 	bss->radius_server_auth_port = 1812;
 	bss->ap_max_inactivity = AP_MAX_INACTIVITY;
+	bss->eapol_version = EAPOL_VERSION;
 }
 
 
@@ -53,15 +56,14 @@ static struct hostapd_config * hostapd_config_defaults(void)
 	struct hostapd_config *conf;
 	struct hostapd_bss_config *bss;
 
-	conf = malloc(sizeof(*conf));
-	bss = malloc(sizeof(*bss) + sizeof(struct hostapd_radius_servers));
+	conf = wpa_zalloc(sizeof(*conf));
+	bss = wpa_zalloc(sizeof(*bss) + sizeof(struct hostapd_radius_servers));
 	if (conf == NULL || bss == NULL) {
 		printf("Failed to allocate memory for configuration data.\n");
 		free(conf);
 		free(bss);
 		return NULL;
 	}
-	memset(conf, 0, sizeof(*conf));
 
 	/* set default driver based on configuration */
 	conf->driver = driver_lookup("default");
@@ -77,6 +79,8 @@ static struct hostapd_config * hostapd_config_defaults(void)
 	conf->num_bss = 1;
 	conf->bss = bss;
 
+	conf->beacon_int = 100;
+
 	return conf;
 }
 
@@ -207,13 +211,12 @@ static int hostapd_config_read_wpa_psk(const char *fname,
 			break;
 		}
 
-		psk = malloc(sizeof(*psk));
+		psk = wpa_zalloc(sizeof(*psk));
 		if (psk == NULL) {
 			printf("WPA PSK allocation failed\n");
 			ret = -1;
 			break;
 		}
-		memset(psk, 0, sizeof(*psk));
 		if (memcmp(addr, "\x00\x00\x00\x00\x00\x00", ETH_ALEN) == 0)
 			psk->group = 1;
 		else
@@ -265,7 +268,7 @@ int hostapd_setup_wpa_psk(struct hostapd_bss_config *conf)
 			       "Using passphrase.\n");
 			free(ssid->wpa_psk);
 		}
-		ssid->wpa_psk = malloc(sizeof(struct hostapd_wpa_psk));
+		ssid->wpa_psk = wpa_zalloc(sizeof(struct hostapd_wpa_psk));
 		if (ssid->wpa_psk == NULL) {
 			printf("Unable to alloc space for PSK\n");
 			return -1;
@@ -275,7 +278,6 @@ int hostapd_setup_wpa_psk(struct hostapd_bss_config *conf)
 		wpa_hexdump_ascii(MSG_DEBUG, "PSK (ASCII passphrase)",
 				  (u8 *) ssid->wpa_passphrase,
 				  strlen(ssid->wpa_passphrase));
-		memset(ssid->wpa_psk, 0, sizeof(struct hostapd_wpa_psk));
 		pbkdf2_sha1(ssid->wpa_passphrase,
 			    ssid->ssid, ssid->ssid_len,
 			    4096, ssid->wpa_psk->psk, PMK_LEN);
@@ -344,12 +346,11 @@ static int hostapd_config_read_eap_user(const char *fname,
 			goto failed;
 		}
 
-		user = malloc(sizeof(*user));
+		user = wpa_zalloc(sizeof(*user));
 		if (user == NULL) {
 			printf("EAP user allocation failed\n");
 			goto failed;
 		}
-		memset(user, 0, sizeof(*user));
 		user->force_version = -1;
 
 		if (buf[0] == '*') {
@@ -734,6 +735,45 @@ static int hostapd_config_read_wep(struct hostapd_wep_keys *wep, int keyidx,
 }
 
 
+static int hostapd_parse_rates(int **rate_list, char *val)
+{
+	int *list;
+	int count;
+	char *pos, *end;
+
+	free(*rate_list);
+	*rate_list = NULL;
+
+	pos = val;
+	count = 0;
+	while (*pos != '\0') {
+		if (*pos == ' ')
+			count++;
+		pos++;
+	}
+
+	list = malloc(sizeof(int) * (count + 2));
+	if (list == NULL)
+		return -1;
+	pos = val;
+	count = 0;
+	while (*pos != '\0') {
+		end = strchr(pos, ' ');
+		if (end)
+			*end = '\0';
+
+		list[count++] = atoi(pos);
+		if (!end)
+			break;
+		pos = end + 1;
+	}
+	list[count] = -1;
+
+	*rate_list = list;
+	return 0;
+}
+
+
 struct hostapd_config * hostapd_config_read(const char *fname)
 {
 	struct hostapd_config *conf;
@@ -855,6 +895,17 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 			bss->assoc_ap = 1;
 		} else if (strcmp(buf, "ieee8021x") == 0) {
 			bss->ieee802_1x = atoi(pos);
+		} else if (strcmp(buf, "eapol_version") == 0) {
+			bss->eapol_version = atoi(pos);
+			if (bss->eapol_version < 1 ||
+			    bss->eapol_version > 2) {
+				printf("Line %d: invalid EAPOL "
+				       "version (%d): '%s'.\n",
+				       line, bss->eapol_version, pos);
+				errors++;
+			} else
+				wpa_printf(MSG_DEBUG, "eapol_version=%d",
+					   bss->eapol_version);
 #ifdef EAP_SERVER
 		} else if (strcmp(buf, "eap_authenticator") == 0) {
 			bss->eap_server = atoi(pos);
@@ -1036,11 +1087,7 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 		} else if (strcmp(buf, "wpa_psk") == 0) {
 			free(bss->ssid.wpa_psk);
 			bss->ssid.wpa_psk =
-				malloc(sizeof(struct hostapd_wpa_psk));
-			if (bss->ssid.wpa_psk) {
-				memset(bss->ssid.wpa_psk, 0,
-				       sizeof(struct hostapd_wpa_psk));
-			}
+				wpa_zalloc(sizeof(struct hostapd_wpa_psk));
 			if (bss->ssid.wpa_psk == NULL)
 				errors++;
 			else if (hexstr2bin(pos, bss->ssid.wpa_psk->psk,
@@ -1097,6 +1144,7 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 			free(bss->ctrl_interface);
 			bss->ctrl_interface = strdup(pos);
 		} else if (strcmp(buf, "ctrl_interface_group") == 0) {
+#ifndef CONFIG_NATIVE_WINDOWS
 			struct group *grp;
 			char *endp;
 			const char *group = pos;
@@ -1122,6 +1170,7 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 			bss->ctrl_interface_gid_set = 1;
 			wpa_printf(MSG_DEBUG, "ctrl_interface_group=%d",
 				   bss->ctrl_interface_gid);
+#endif /* CONFIG_NATIVE_WINDOWS */
 #ifdef RADIUS_SERVER
 		} else if (strcmp(buf, "radius_server_clients") == 0) {
 			free(bss->radius_server_clients);
@@ -1136,6 +1185,46 @@ struct hostapd_config * hostapd_config_read(const char *fname)
 			bss->test_socket = strdup(pos);
 		} else if (strcmp(buf, "use_pae_group_addr") == 0) {
 			bss->use_pae_group_addr = atoi(pos);
+		} else if (strcmp(buf, "hw_mode") == 0) {
+			if (strcmp(pos, "a") == 0)
+				conf->hw_mode = HOSTAPD_MODE_IEEE80211A;
+			else if (strcmp(pos, "b") == 0)
+				conf->hw_mode = HOSTAPD_MODE_IEEE80211B;
+			else if (strcmp(pos, "g") == 0)
+				conf->hw_mode = HOSTAPD_MODE_IEEE80211G;
+			else {
+				printf("Line %d: unknown hw_mode '%s'\n",
+				       line, pos);
+				errors++;
+			}
+		} else if (strcmp(buf, "channel") == 0) {
+			conf->channel = atoi(pos);
+		} else if (strcmp(buf, "beacon_int") == 0) {
+			int val = atoi(pos);
+			/* MIB defines range as 1..65535, but very small values
+			 * cause problems with the current implementation.
+			 * Since it is unlikely that this small numbers are
+			 * useful in real life scenarios, do not allow beacon
+			 * period to be set below 15 TU. */
+			if (val < 15 || val > 65535) {
+				printf("Line %d: invalid beacon_int %d "
+				       "(expected 15..65535)\n",
+				       line, val);
+				errors++;
+			} else
+				conf->beacon_int = val;
+		} else if (strcmp(buf, "supported_rates") == 0) {
+			if (hostapd_parse_rates(&conf->supported_rates, pos)) {
+				printf("Line %d: invalid rate list\n", line);
+				errors++;
+			}
+		} else if (strcmp(buf, "basic_rates") == 0) {
+			if (hostapd_parse_rates(&conf->basic_rates, pos)) {
+				printf("Line %d: invalid rate list\n", line);
+				errors++;
+			}
+		} else if (strcmp(buf, "ignore_broadcast_ssid") == 0) {
+			bss->ignore_broadcast_ssid = atoi(pos);
 		} else if (strcmp(buf, "wep_default_key") == 0) {
 			bss->ssid.wep.idx = atoi(pos);
 			if (bss->ssid.wep.idx > 3) {
@@ -1302,6 +1391,21 @@ int hostapd_maclist_found(macaddr *list, int num_entries, const u8 *addr)
 }
 
 
+int hostapd_rate_found(int *list, int rate)
+{
+	int i;
+
+	if (list == NULL)
+		return 0;
+
+	for (i = 0; list[i] >= 0; i++)
+		if (list[i] == rate)
+			return 1;
+
+	return 0;
+}
+
+
 const u8 * hostapd_get_psk(const struct hostapd_bss_config *conf,
 			   const u8 *addr, const u8 *prev_psk)
 {
