@@ -1,6 +1,6 @@
 /*
  * hostapd / Configuration file
- * Copyright (c) 2003-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2003-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -118,6 +118,7 @@ struct hostapd_bss_config {
 	char *dump_log_name; /* file name for state dump (SIGUSR1) */
 
 	int ieee802_1x; /* use IEEE 802.1X */
+	int eapol_version;
 	int eap_server; /* Use internal EAP server instead of external
 			 * RADIUS server */
 	struct hostapd_eap_user *eap_user;
@@ -202,9 +203,18 @@ struct hostapd_bss_config {
 				 */
 
 	int ap_max_inactivity;
+	int ignore_broadcast_ssid;
 };
 
 
+typedef enum {
+	HOSTAPD_MODE_IEEE80211B,
+	HOSTAPD_MODE_IEEE80211G,
+	HOSTAPD_MODE_IEEE80211A,
+	NUM_HOSTAPD_MODES
+} hostapd_hw_mode;
+
+
 /**
  * struct hostapd_config - Per-radio interface configuration
  */
@@ -212,6 +222,13 @@ struct hostapd_config {
 	struct hostapd_bss_config *bss, *last_bss;
 	size_t num_bss;
 
+	u16 beacon_int;
+	u8 channel;
+	hostapd_hw_mode hw_mode; /* HOSTAPD_MODE_IEEE80211A, .. */
+
+	int *supported_rates;
+	int *basic_rates;
+
 	const struct driver_ops *driver;
 };
 
@@ -219,6 +236,7 @@ struct hostapd_config {
 struct hostapd_config * hostapd_config_read(const char *fname);
 void hostapd_config_free(struct hostapd_config *conf);
 int hostapd_maclist_found(macaddr *list, int num_entries, const u8 *addr);
+int hostapd_rate_found(int *list, int rate);
 const u8 * hostapd_get_psk(const struct hostapd_bss_config *conf,
 			   const u8 *addr, const u8 *prev_psk);
 int hostapd_setup_wpa_psk(struct hostapd_bss_config *conf);
