@@ -318,6 +318,17 @@ bsd_set_sta_authorized(void *priv, const u8 *addr, int authorized)
 	return set80211var(priv, IEEE80211_IOC_MLME, &mlme, sizeof(mlme));
 }
 
+static int
+bsd_sta_set_flags(void *priv, const u8 *addr, int flags_or, int flags_and)
+{
+	/* For now, only support setting Authorized flag */
+	if (flags_or & WLAN_STA_AUTHORIZED)
+		return bsd_set_sta_authorized(priv, addr, 1);
+	if (flags_and & WLAN_STA_AUTHORIZED)
+		return bsd_set_sta_authorized(priv, addr, 0);
+	return 0;
+}
+
 static int
 bsd_del_key(void *priv, const u8 *addr, int key_idx)
 {
@@ -774,13 +785,12 @@ bsd_init(struct hostapd_data *hapd)
 {
 	struct bsd_driver_data *drv;
 
-	drv = malloc(sizeof(struct bsd_driver_data));
+	drv = wpa_zalloc(sizeof(struct bsd_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for bsd driver data\n");
 		goto bad;
 	}
 
-	memset(drv, 0, sizeof(*drv));
 	drv->ops = bsd_driver_ops;
 	drv->hapd = hapd;
 	drv->ioctl_sock = socket(PF_INET, SOCK_DGRAM, 0);
@@ -839,7 +849,7 @@ static const struct driver_ops bsd_driver_ops = {
 	.set_generic_elem	= bsd_set_opt_ie,
 	.wireless_event_init	= bsd_wireless_event_init,
 	.wireless_event_deinit	= bsd_wireless_event_deinit,
-	.set_sta_authorized	= bsd_set_sta_authorized,
+	.sta_set_flags		= bsd_sta_set_flags,
 	.read_sta_data		= bsd_read_sta_driver_data,
 	.send_eapol		= bsd_send_eapol,
 	.sta_disassoc		= bsd_sta_disassoc,
