@@ -36,8 +36,10 @@
 #include "priv_netlink.h"
 #include "ieee802_11.h"
 #include "sta_info.h"
+#include "hw_features.h"
 #include <hostapd_ioctl.h>
 #include <net/d80211_common.h>
+#include <net/d80211_shared.h>
 
 
 /* from net/d80211.h */
@@ -53,6 +55,7 @@
 #define WLAN_FC_TYPE_CTRL	1
 #define WLAN_FC_TYPE_DATA	2
 
+#define WLAN_FC_STYPE_PROBE_REQ 4
 #define WLAN_FC_STYPE_BEACON		8
 #define WLAN_FC_STYPE_DISASSOC		10
 #define WLAN_FC_STYPE_DEAUTH		12
@@ -76,12 +79,13 @@ struct i802_driver_data {
 	int we_version;
 };
 
-static const struct driver_ops i802_driver_ops;
+static const struct driver_ops devicescape_driver_ops;
 
 
 #define HAPD_DECL	struct hostapd_data *hapd = iface->bss[0]
 
-static int i802_set_sta_authorized(void *priv, const u8 *addr, int authorized);
+static int i802_sta_set_flags(void *priv, const u8 *addr,
+			      int flags_or, int flags_and);
 
 
 static int hostapd_set_iface_flags(struct i802_driver_data *drv, int dev_up)
@@ -179,10 +183,9 @@ static int i802_set_encryption(const char *iface, void *priv, const char *alg,
 	int ret = 0;
 
 	blen = sizeof(*param) + key_len;
-	buf = malloc(blen);
+	buf = wpa_zalloc(blen);
 	if (buf == NULL)
 		return -1;
-	memset(buf, 0, blen);
 
 	param = (struct prism2_hostapd_param *) buf;
 	param->cmd = PRISM2_SET_ENCRYPTION;
@@ -218,11 +221,10 @@ static int i802_get_seqnum(const char *iface, void *priv, const u8 *addr,
 	int ret;
 
 	param_len = sizeof(struct prism2_hostapd_param) + 32;
-	param = (struct prism2_hostapd_param *) malloc(param_len);
+	param = wpa_zalloc(param_len);
 	if (param == NULL)
 		return -1;
 
-	memset(param, 0, param_len);
 	param->cmd = PRISM2_GET_ENCRYPTION;
 	if (addr == NULL)
 		memset(param->sta_addr, 0xff, ETH_ALEN);
@@ -240,6 +242,51 @@ static int i802_get_seqnum(const char *iface, void *priv, const u8 *addr,
 }
 
 
+static int i802_set_rate_sets(void *priv, int *supp_rates, int *basic_rates,
+			      int mode)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param *param;
+	u8 *buf;
+	u16 *pos;
+	size_t blen;
+	int ret = 0;
+	int i, num_supp = 0, num_basic = 0;
+
+	if (supp_rates)
+		for (i = 0; supp_rates[i] >= 0; i++)
+			num_supp++;
+	if (basic_rates)
+		for (i = 0; basic_rates[i] >= 0; i++)
+			num_basic++;
+
+	blen = sizeof(*param) + (num_supp + num_basic) * 2;
+	buf = wpa_zalloc(blen);
+	if (buf == NULL)
+		return -1;
+
+	param = (struct prism2_hostapd_param *) buf;
+	param->cmd = PRISM2_HOSTAPD_SET_RATE_SETS;
+	param->u.set_rate_sets.mode = mode;
+	param->u.set_rate_sets.num_supported_rates = num_supp;
+	param->u.set_rate_sets.num_basic_rates = num_basic;
+	pos = (u16 *) param->u.set_rate_sets.data;
+
+	for (i = 0; i < num_supp; i++)
+		*pos++ = supp_rates[i];
+	for (i = 0; i < num_basic; i++)
+		*pos++ = basic_rates[i];
+
+	if (hostapd_ioctl(drv, param, blen)) {
+		printf("Failed to set rate sets.\n");
+		ret = -1;
+	}
+	free(buf);
+
+	return ret;
+}
+
+
 static int hostap_ioctl_prism2param_iface(const char *iface,
 					  struct i802_driver_data *drv,
 					  int param, int value)
@@ -302,6 +349,156 @@ static int i802_send_mgmt_frame(void *priv, const void *msg, size_t len,
 }
 
 
+/* Set kernel driver on given frequency (MHz) */
+static int i802_set_freq(void *priv, int mode, int freq)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	hostap_ioctl_prism2param(drv, PRISM2_PARAM_NEXT_MODE, mode);
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+	iwr.u.freq.m = freq;
+	iwr.u.freq.e = 6;
+
+	if (ioctl(drv->ioctl_sock, SIOCSIWFREQ, &iwr) < 0) {
+		perror("ioctl[SIOCSIWFREQ]");
+		return -1;
+	}
+
+	return 0;
+}
+
+
+static int i802_set_rts(void *priv, int rts)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+	iwr.u.rts.value = rts;
+	iwr.u.rts.fixed = 1;
+
+	if (ioctl(drv->ioctl_sock, SIOCSIWRTS, &iwr) < 0) {
+		perror("ioctl[SIOCSIWRTS]");
+		return -1;
+	}
+
+	return 0;
+}
+
+
+static int i802_get_rts(void *priv, int *rts)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+
+	if (ioctl(drv->ioctl_sock, SIOCGIWRTS, &iwr) < 0) {
+		perror("ioctl[SIOCGIWRTS]");
+		return -1;
+	}
+
+	*rts = iwr.u.rts.value;
+
+	return 0;
+}
+
+
+static int i802_set_frag(void *priv, int frag)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+	iwr.u.frag.value = frag;
+	iwr.u.frag.fixed = 1;
+
+	if (ioctl(drv->ioctl_sock, SIOCSIWFRAG, &iwr) < 0) {
+		perror("ioctl[SIOCSIWFRAG]");
+		return -1;
+	}
+
+	return 0;
+}
+
+
+static int i802_get_frag(void *priv, int *frag)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+
+	if (ioctl(drv->ioctl_sock, SIOCGIWFRAG, &iwr) < 0) {
+		perror("ioctl[SIOCGIWFRAG]");
+		return -1;
+	}
+
+	*frag = iwr.u.frag.value;
+
+	return 0;
+}
+
+
+static int i802_set_retry(void *priv, int short_retry, int long_retry)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+
+	iwr.u.retry.value = short_retry;
+	iwr.u.retry.flags = IW_RETRY_LIMIT | IW_RETRY_MIN;
+	if (ioctl(drv->ioctl_sock, SIOCSIWFRAG, &iwr) < 0) {
+		perror("ioctl[SIOCSIWRETRY(short)]");
+		return -1;
+	}
+
+	iwr.u.retry.value = long_retry;
+	iwr.u.retry.flags = IW_RETRY_LIMIT | IW_RETRY_MAX;
+	if (ioctl(drv->ioctl_sock, SIOCSIWFRAG, &iwr) < 0) {
+		perror("ioctl[SIOCSIWRETRY(long)]");
+		return -1;
+	}
+
+	return 0;
+}
+
+
+static int i802_get_retry(void *priv, int *short_retry, int *long_retry)
+{
+	struct i802_driver_data *drv = priv;
+	struct iwreq iwr;
+
+	memset(&iwr, 0, sizeof(iwr));
+	strncpy(iwr.ifr_name, drv->hapd->conf->iface, IFNAMSIZ);
+
+	iwr.u.retry.flags = IW_RETRY_LIMIT | IW_RETRY_MIN;
+	if (ioctl(drv->ioctl_sock, SIOCGIWRETRY, &iwr) < 0) {
+		perror("ioctl[SIOCGIWFRAG(short)]");
+		return -1;
+	}
+	*short_retry = iwr.u.retry.value;
+
+	iwr.u.retry.flags = IW_RETRY_LIMIT | IW_RETRY_MAX;
+	if (ioctl(drv->ioctl_sock, SIOCGIWRETRY, &iwr) < 0) {
+		perror("ioctl[SIOCGIWFRAG(long)]");
+		return -1;
+	}
+	*long_retry = iwr.u.retry.value;
+
+	return 0;
+}
+
+
 static int i802_flush(void *priv)
 {
 	struct i802_driver_data *drv = priv;
@@ -361,14 +558,13 @@ static int i802_send_eapol(void *priv, const u8 *addr, const u8 *data,
 
 	len = sizeof(*hdr) + (qos ? 2 : 0) + sizeof(rfc1042_header) + 2 +
 		data_len;
-	hdr = malloc(len);
+	hdr = wpa_zalloc(len);
 	if (hdr == NULL) {
 		printf("malloc() failed for i802_send_data(len=%lu)\n",
 		       (unsigned long) len);
 		return -1;
 	}
 
-	memset(hdr, 0, len);
 	hdr->frame_control =
 		IEEE80211_FC(WLAN_FC_TYPE_DATA, WLAN_FC_STYPE_DATA);
 	hdr->frame_control |= host_to_le16(WLAN_FC_FROMDS);
@@ -432,19 +628,22 @@ static int i802_set_assoc_ap(void *priv, const u8 *addr)
 
 
 static int i802_sta_add(const char *ifname, void *priv, const u8 *addr,
-			u16 aid, u16 capability, u8 tx_supp_rates)
+			u16 aid, u16 capability, u8 *supp_rates,
+			size_t supp_rates_len, int flags)
 {
 	struct i802_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
+	int len;
 
 	memset(&param, 0, sizeof(param));
 	param.cmd = PRISM2_HOSTAPD_ADD_STA;
 	memcpy(param.sta_addr, addr, ETH_ALEN);
 	param.u.add_sta.aid = aid;
 	param.u.add_sta.capability = capability;
-#if 0 /* FIX: new TX supp rates format */
-	param.u.add_sta.tx_supp_rates = tx_supp_rates;
-#endif
+	len = supp_rates_len;
+	if (len > sizeof(param.u.add_sta.supp_rates))
+		len = sizeof(param.u.add_sta.supp_rates);
+	memcpy(param.u.add_sta.supp_rates, supp_rates, len);
 	return hostapd_ioctl_iface(ifname, drv, &param, sizeof(param));
 }
 
@@ -454,7 +653,7 @@ static int i802_sta_remove(void *priv, const u8 *addr)
 	struct i802_driver_data *drv = priv;
 	struct prism2_hostapd_param param;
 
-	i802_set_sta_authorized(drv, addr, 0);
+	i802_sta_set_flags(drv, addr, 0, ~WLAN_STA_AUTHORIZED);
 
 	memset(&param, 0, sizeof(param));
 	param.cmd = PRISM2_HOSTAPD_REMOVE_STA;
@@ -465,6 +664,21 @@ static int i802_sta_remove(void *priv, const u8 *addr)
 }
 
 
+static int i802_sta_set_flags(void *priv, const u8 *addr,
+			      int flags_or, int flags_and)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param param;
+
+	memset(&param, 0, sizeof(param));
+	param.cmd = PRISM2_HOSTAPD_SET_FLAGS_STA;
+	memcpy(param.sta_addr, addr, ETH_ALEN);
+	param.u.set_flags_sta.flags_or = flags_or;
+	param.u.set_flags_sta.flags_and = flags_and;
+	return hostapd_ioctl(drv, &param, sizeof(param));
+}
+
+
 static int i802_set_generic_elem(void *priv, const u8 *elem, size_t elem_len)
 {
 	struct i802_driver_data *drv = priv;
@@ -474,11 +688,11 @@ static int i802_set_generic_elem(void *priv, const u8 *elem, size_t elem_len)
 	int ret = 0;
 
 	blen = sizeof(*param) + elem_len;
-	buf = malloc(blen);
+	buf = wpa_zalloc(blen);
 	if (buf == NULL)
 		return -1;
-	memset(buf, 0, blen);
 
+	param = (struct prism2_hostapd_param *) buf;
 	param->cmd = PRISM2_HOSTAPD_SET_GENERIC_INFO_ELEM;
 	param->u.set_generic_info_elem.len = elem_len;
 	memcpy(param->u.set_generic_info_elem.data, elem, elem_len);
@@ -493,6 +707,116 @@ static int i802_set_generic_elem(void *priv, const u8 *elem, size_t elem_len)
 }
 
 
+static int i802_set_channel_flag(void *priv, int mode, int chan, int flag,
+				 unsigned char power_level,
+				 unsigned char antenna_max)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param param;
+
+	memset(&param, 0, sizeof(param));
+	param.cmd = PRISM2_HOSTAPD_SET_CHANNEL_FLAG;
+	switch (mode) {
+	case HOSTAPD_MODE_IEEE80211A:
+		param.u.set_channel_flag.mode = MODE_IEEE80211A;
+		break;
+	case HOSTAPD_MODE_IEEE80211B:
+		param.u.set_channel_flag.mode = MODE_IEEE80211B;
+		break;
+	case HOSTAPD_MODE_IEEE80211G:
+		param.u.set_channel_flag.mode = MODE_IEEE80211G;
+		break;
+	}
+
+	param.u.set_channel_flag.chan = chan;
+	param.u.set_channel_flag.flag = flag;
+	param.u.set_channel_flag.power_level = power_level;
+	param.u.set_channel_flag.antenna_max = antenna_max;
+
+	if (hostapd_ioctl(drv, &param, sizeof(param))) {
+		printf("Failed to set channel flag (mode=%d chan=%d flag=0x%x)"
+		       "\n", mode, chan, flag);
+		return -1;
+	}
+
+	return 0;
+}
+
+
+static int i802_set_regulatory_domain(void *priv, unsigned int rd)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param param;
+
+	memset(&param, 0, sizeof(param));
+	param.cmd = PRISM2_HOSTAPD_SET_REGULATORY_DOMAIN;
+	param.u.set_regulatory_domain.rd = rd;
+
+	if (hostapd_ioctl(drv, &param, sizeof(param))) {
+		printf("Failed to set regulatory domain (%x)\n", rd);
+		return -1;
+	}
+
+	return 0;
+}
+
+
+static int i802_set_tx_queue_params(void *priv, int queue, int aifs,
+				    int cw_min, int cw_max, int burst_time)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param param;
+
+	memset(&param, 0, sizeof(param));
+	param.cmd = PRISM2_HOSTAPD_SET_TX_QUEUE_PARAMS;
+	param.u.tx_queue_params.queue = queue;
+	param.u.tx_queue_params.aifs = aifs;
+	param.u.tx_queue_params.cw_min = cw_min;
+	param.u.tx_queue_params.cw_max = cw_max;
+	param.u.tx_queue_params.burst_time = burst_time;
+
+	if (hostapd_ioctl(drv, &param, sizeof(param)))
+		return -1;
+
+	return 0;
+}
+
+
+static int i802_set_beacon(const char *ifname, void *priv,
+			   u8 *head, size_t head_len,
+			   u8 *tail, size_t tail_len)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param *param;
+	int len, ret = 0;
+
+	param = wpa_zalloc(sizeof(*param) + head_len + tail_len);
+	if (param == NULL) {
+		printf("Failed to alloc memory for beacon ioctl\n");
+		return -1;
+	}
+	len = (&param->u.beacon.data[0] - (u8 *) param) + head_len + tail_len;
+	param->cmd = PRISM2_HOSTAPD_SET_BEACON;
+	param->u.beacon.head_len = head_len;
+	param->u.beacon.tail_len = tail_len;
+	memcpy(&param->u.beacon.data[0], head, head_len);
+	memcpy(&param->u.beacon.data[0] + head_len, tail, tail_len);
+
+	if (len < sizeof(*param))
+		len = sizeof(*param);
+	if (hostapd_ioctl_iface(ifname, drv, param, len)) {
+		printf("Could not set beacon data to kernel driver.\n");
+		printf("ifname='%s' head=%p head_len=%d tail=%p tail_len=%d "
+		       "cmd=%d\n",
+		       ifname, head, head_len, tail, tail_len, param->cmd);
+		ret = -1;
+	}
+
+	free(param);
+	return ret;
+}
+
+
 static int i802_set_ieee8021x(const char *ifname, void *priv, int enabled)
 {
 	struct i802_driver_data *drv = priv;
@@ -523,6 +847,161 @@ static int i802_set_privacy(void *priv, int enabled)
 }
 
 
+static int i802_set_beacon_int(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv, PRISM2_PARAM_BEACON_INT, value);
+}
+
+
+static int i802_set_dtim_period(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv, PRISM2_PARAM_DTIM_PERIOD, value);
+}
+
+
+static int i802_set_broadcast_ssid(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv, PRISM2_PARAM_BROADCAST_SSID,
+					value);
+}
+
+
+static int i802_set_cts_protect(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv,
+					PRISM2_PARAM_CTS_PROTECT_ERP_FRAMES,
+					value);
+}
+
+
+static int i802_set_key_tx_rx_threshold(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv, PRISM2_PARAM_KEY_TX_RX_THRESHOLD,
+					value);
+}
+
+
+static int i802_set_preamble(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv, PRISM2_PARAM_PREAMBLE, value);
+}
+
+
+static int i802_set_short_slot_time(void *priv, int value)
+{
+	struct i802_driver_data *drv = priv;
+	return hostap_ioctl_prism2param(drv, PRISM2_PARAM_SHORT_SLOT_TIME,
+					value);
+}
+
+
+static struct hostapd_hw_modes * i802_get_hw_feature_data(void *priv,
+							  u16 *num_modes,
+							  u16 *flags)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param *param;
+	u8 *pos, *end;
+	struct hostapd_hw_modes *modes;
+	int i;
+
+	param = wpa_zalloc(PRISM2_HOSTAPD_MAX_BUF_SIZE);
+	if (param == NULL)
+		return NULL;
+	param->cmd = PRISM2_HOSTAPD_GET_HW_FEATURES;
+	if (hostapd_ioctl(drv, param, PRISM2_HOSTAPD_MAX_BUF_SIZE)) {
+		free(param);
+		return NULL;
+	}
+
+	*num_modes = param->u.hw_features.num_modes;
+	*flags = param->u.hw_features.flags;
+
+	pos = param->u.hw_features.data;
+	end = pos + PRISM2_HOSTAPD_MAX_BUF_SIZE -
+		(param->u.hw_features.data - (u8 *) param);
+
+	modes = wpa_zalloc(*num_modes * sizeof(struct hostapd_hw_modes));
+	if (modes == NULL) {
+		free(param);
+		return NULL;
+	}
+
+	for (i = 0; i < *num_modes; i++) {
+		struct hostapd_ioctl_hw_modes_hdr *hdr;
+		struct hostapd_hw_modes *feature;
+		int clen, rlen;
+
+		hdr = (struct hostapd_ioctl_hw_modes_hdr *) pos;
+		feature = &modes[i];
+		switch (hdr->mode) {
+		case MODE_IEEE80211A:
+			feature->mode = HOSTAPD_MODE_IEEE80211A;
+			break;
+		case MODE_IEEE80211B:
+			feature->mode = HOSTAPD_MODE_IEEE80211B;
+			break;
+		case MODE_IEEE80211G:
+			feature->mode = HOSTAPD_MODE_IEEE80211G;
+			break;
+		default:
+			wpa_printf(MSG_ERROR, "Unknown hw_mode=%d in "
+				   "get_hw_features data", hdr->mode);
+			hostapd_free_hw_features(modes, *num_modes);
+			modes = NULL;
+			break;
+		}
+		feature->num_channels = hdr->num_channels;
+		feature->num_rates = hdr->num_rates;
+
+		pos = (u8 *) (hdr + 1);
+
+		clen = hdr->num_channels *
+			sizeof(struct hostapd_channel_data);
+		rlen = hdr->num_rates *
+			sizeof(struct hostapd_rate_data);
+
+		feature->channels = malloc(clen);
+		feature->rates = malloc(rlen);
+		if (!feature->channels || !feature->rates) {
+			hostapd_free_hw_features(modes, *num_modes);
+			modes = NULL;
+			break;
+		}
+
+		memcpy(feature->channels, pos, clen);
+		pos += clen;
+		memcpy(feature->rates, pos, rlen);
+		pos += rlen;
+	}
+
+	free(param);
+
+	return modes;
+}
+
+
+static int i802_set_sta_vlan(void *priv, const u8 *addr, const char *ifname,
+			     int vlan_id)
+{
+	struct i802_driver_data *drv = priv;
+	struct prism2_hostapd_param param;
+
+	memset(&param, 0, sizeof(param));
+	param.cmd = PRISM2_HOSTAPD_SET_STA_VLAN;
+	memcpy(param.sta_addr, addr, ETH_ALEN);
+	strncpy(param.u.set_sta_vlan.vlan_name, ifname, IFNAMSIZ);
+	param.u.set_sta_vlan.vlan_id = vlan_id;
+	return hostapd_ioctl(drv, &param, sizeof(param));
+}
+
+
 static void handle_data(struct hostapd_data *hapd, u8 *buf, size_t len,
 			u16 stype, struct ieee80211_frame_info *fi)
 {
@@ -789,7 +1268,8 @@ static void handle_frame(struct hostapd_iface *iface, u8 *buf, size_t len,
 
 	switch (type) {
 	case WLAN_FC_TYPE_MGMT:
-		HOSTAPD_DEBUG(stype == WLAN_FC_STYPE_BEACON ?
+		HOSTAPD_DEBUG((stype == WLAN_FC_STYPE_BEACON ||
+			       stype == WLAN_FC_STYPE_PROBE_REQ) ?
 			      HOSTAPD_DEBUG_EXCESSIVE : HOSTAPD_DEBUG_VERBOSE,
 			      "MGMT\n");
 		if (broadcast_bssid) {
@@ -926,29 +1406,6 @@ static int i802_init_sockets(struct i802_driver_data *drv)
 }
 
 
-static int i802_set_sta_authorized(void *priv, const u8 *addr, int authorized)
-{
-	struct i802_driver_data *drv = priv;
-	struct prism2_hostapd_param param;
-
-	memset(&param, 0, sizeof(param));
-	param.cmd = PRISM2_HOSTAPD_SET_FLAGS_STA;
-	memcpy(param.sta_addr, addr, ETH_ALEN);
-	if (authorized) {
-		param.u.set_flags_sta.flags_or = WLAN_STA_AUTHORIZED;
-		param.u.set_flags_sta.flags_and = ~0;
-	} else {
-		param.u.set_flags_sta.flags_and = ~WLAN_STA_AUTHORIZED;
-	}
-
-	if (hostapd_ioctl(drv, &param, sizeof(param))) {
-		printf("Could not set station flags for kernel driver.\n");
-		return -1;
-	}
-	return 0;
-}
-
-
 static int i802_get_inact_sec(void *priv, const u8 *addr)
 {
 	struct i802_driver_data *drv = priv;
@@ -1155,10 +1612,9 @@ static int hostap_get_we_version(struct i802_driver_data *drv)
 	 * structure to grow in the future.
 	 */
 	buflen = sizeof(struct iw_range) + 500;
-	range = malloc(buflen);
+	range = wpa_zalloc(buflen);
 	if (range == NULL)
 		return -1;
-	memset(range, 0, buflen);
 
 	memset(&iwr, 0, sizeof(iwr));
 	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
@@ -1268,14 +1724,13 @@ static int i802_init(struct hostapd_data *hapd)
 {
 	struct i802_driver_data *drv;
 
-	drv = malloc(sizeof(struct i802_driver_data));
+	drv = wpa_zalloc(sizeof(struct i802_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for i802 driver data\n");
 		return -1;
 	}
 
-	memset(drv, 0, sizeof(*drv));
-	drv->ops = i802_driver_ops;
+	drv->ops = devicescape_driver_ops;
 	drv->hapd = hapd;
 	memcpy(drv->iface, hapd->conf->iface, sizeof(drv->iface));
 
@@ -1284,6 +1739,9 @@ static int i802_init(struct hostapd_data *hapd)
 
 	hapd->driver = &drv->ops;
 
+	/* Enable the radio by default. */
+	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_ADM_STATUS, 1);
+
 	return 0;
 
 failed:
@@ -1296,6 +1754,9 @@ static void i802_deinit(void *priv)
 {
 	struct i802_driver_data *drv = priv;
 
+	/* Disable the radio. */
+	(void) hostap_ioctl_prism2param(drv, PRISM2_PARAM_ADM_STATUS, 0);
+
 	drv->hapd->driver = NULL;
 
 	(void) hostapd_set_iface_flags(drv, 0);
@@ -1323,7 +1784,7 @@ static const struct driver_ops devicescape_driver_ops = {
 	.set_generic_elem = i802_set_generic_elem,
 	.read_sta_data = i802_read_sta_data,
 	.send_eapol = i802_send_eapol,
-	.set_sta_authorized = i802_set_sta_authorized,
+	.sta_set_flags = i802_sta_set_flags,
 	.sta_deauth = i802_sta_deauth,
 	.sta_disassoc = i802_sta_disassoc,
 	.sta_remove = i802_sta_remove,
@@ -1333,6 +1794,27 @@ static const struct driver_ops devicescape_driver_ops = {
 	.sta_add = i802_sta_add,
 	.get_inact_sec = i802_get_inact_sec,
 	.sta_clear_stats = i802_sta_clear_stats,
+	.set_freq = i802_set_freq,
+	.set_rts = i802_set_rts,
+	.get_rts = i802_get_rts,
+	.set_frag = i802_set_frag,
+	.get_frag = i802_get_frag,
+	.set_retry = i802_set_retry,
+	.get_retry = i802_get_retry,
+	.set_rate_sets = i802_set_rate_sets,
+	.set_channel_flag = i802_set_channel_flag,
+	.set_regulatory_domain = i802_set_regulatory_domain,
+	.set_beacon = i802_set_beacon,
+	.set_beacon_int = i802_set_beacon_int,
+	.set_dtim_period = i802_set_dtim_period,
+	.set_broadcast_ssid = i802_set_broadcast_ssid,
+	.set_cts_protect = i802_set_cts_protect,
+	.set_key_tx_rx_threshold = i802_set_key_tx_rx_threshold,
+	.set_preamble = i802_set_preamble,
+	.set_short_slot_time = i802_set_short_slot_time,
+	.set_tx_queue_params = i802_set_tx_queue_params,
+	.get_hw_feature_data = i802_get_hw_feature_data,
+	.set_sta_vlan = i802_set_sta_vlan,
 };
 
 
