@@ -399,6 +399,17 @@ madwifi_set_sta_authorized(void *priv, const u8 *addr, int authorized)
 			    sizeof(mlme));
 }
 
+static int
+madwifi_sta_set_flags(void *priv, const u8 *addr, int flags_or, int flags_and)
+{
+	/* For now, only support setting Authorized flag */
+	if (flags_or & WLAN_STA_AUTHORIZED)
+		return madwifi_set_sta_authorized(priv, addr, 1);
+	if (flags_and & WLAN_STA_AUTHORIZED)
+		return madwifi_set_sta_authorized(priv, addr, 0);
+	return 0;
+}
+
 static int
 madwifi_del_key(void *priv, const u8 *addr, int key_idx)
 {
@@ -997,10 +1008,9 @@ madwifi_get_we_version(struct madwifi_driver_data *drv)
 	 * structure to grow in the future.
 	 */
 	buflen = sizeof(struct iw_range) + 500;
-	range = malloc(buflen);
+	range = wpa_zalloc(buflen);
 	if (range == NULL)
 		return -1;
-	memset(range, 0, buflen);
 
 	memset(&iwr, 0, sizeof(iwr));
 	strncpy(iwr.ifr_name, drv->iface, IFNAMSIZ);
@@ -1142,13 +1152,12 @@ madwifi_init(struct hostapd_data *hapd)
 	struct ifreq ifr;
 	struct iwreq iwr;
 
-	drv = malloc(sizeof(struct madwifi_driver_data));
+	drv = wpa_zalloc(sizeof(struct madwifi_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for madwifi driver data\n");
 		goto bad;
 	}
 
-	memset(drv, 0, sizeof(*drv));
 	drv->ops = madwifi_driver_ops;
 	drv->hapd = hapd;
 	drv->ioctl_sock = socket(PF_INET, SOCK_DGRAM, 0);
@@ -1288,7 +1297,7 @@ static const struct driver_ops madwifi_driver_ops = {
 	.set_generic_elem	= madwifi_set_opt_ie,
 	.wireless_event_init	= madwifi_wireless_event_init,
 	.wireless_event_deinit	= madwifi_wireless_event_deinit,
-	.set_sta_authorized	= madwifi_set_sta_authorized,
+	.sta_set_flags		= madwifi_sta_set_flags,
 	.read_sta_data		= madwifi_read_sta_driver_data,
 	.send_eapol		= madwifi_send_eapol,
 	.sta_disassoc		= madwifi_sta_disassoc,
