@@ -137,15 +137,13 @@ static int prism54_send_eapol(void *priv, const u8 *addr,
 	int res;
 
 	len = sizeof(*hdr) + data_len;
-	hdr = malloc(len);
+	hdr = wpa_zalloc(len);
 	if (hdr == NULL) {
-		printf("malloc() failed for hostapd_send_data(len=%lu)\n",
+		printf("malloc() failed for prism54_send_data(len=%lu)\n",
 		       (unsigned long) len);
 		return -1;
 	}
 
-	memset(hdr, 0, len);
-
 	memcpy(&hdr->da[0], addr, ETH_ALEN);
 	memcpy(&hdr->sa[0], drv->hapd->own_addr, ETH_ALEN);
 	hdr->type = htons(ETH_P_PAE);
@@ -191,6 +189,18 @@ static int prism54_set_sta_authorized(void *priv, const u8 *addr,
 }
 
 
+static int
+prism54_sta_set_flags(void *priv, const u8 *addr, int flags_or, int flags_and)
+{
+	/* For now, only support setting Authorized flag */
+	if (flags_or & WLAN_STA_AUTHORIZED)
+		return prism54_set_sta_authorized(priv, addr, 1);
+	if (flags_and & WLAN_STA_AUTHORIZED)
+		return prism54_set_sta_authorized(priv, addr, 0);
+	return 0;
+}
+
+
 /* set per station key */
 static int prism54_set_encryption(const char *ifname, void *priv,
 				  const char *alg, const u8 *addr,
@@ -531,12 +541,11 @@ static int prism54_set_generic_elem(void *priv,
 	char *pos;
 	struct obj_attachment_hdr *attach;
 	size_t blen = sizeof(*hdr) + sizeof(*attach) + elem_len;
-	hdr = malloc(blen);
+	hdr = wpa_zalloc(blen);
 	if (hdr == NULL) {
 		printf("%s: memory low\n", __func__);
 		return -1;
 	}
-	memset(hdr, 0, blen);
 	hdr->op = htonl(PIMOP_SET);
 	hdr->oid = htonl(DOT11_OID_ATTACHMENT);
 	attach = (struct obj_attachment_hdr *)&hdr[1];
@@ -994,14 +1003,13 @@ static int prism54_driver_init(struct hostapd_data *hapd)
 {
 	struct prism54_driver_data *drv;
 
-	drv = malloc(sizeof(struct prism54_driver_data));
+	drv = wpa_zalloc(sizeof(struct prism54_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for hostapd Prism54 driver "
 		       "data\n");
 		return -1;
 	}
 
-	memset(drv, 0, sizeof(*drv));
 	drv->ops = prism54_driver_ops;
 	drv->hapd = hapd;
 	drv->pim_sock = drv->sock = -1;
@@ -1045,7 +1053,7 @@ static const struct driver_ops prism54_driver_ops = {
 	.flush = prism54_flush,
 	.set_generic_elem = prism54_set_generic_elem,
 	.send_eapol = prism54_send_eapol,
-	.set_sta_authorized = prism54_set_sta_authorized,
+	.sta_set_flags = prism54_sta_set_flags,
 	.sta_deauth = prism54_sta_deauth,
 	.sta_disassoc = prism54_sta_disassoc,
 	.set_ssid = prism54_ioctl_setiwessid,
