@@ -209,11 +209,10 @@ static void test_driver_assoc(struct test_driver_data *drv,
 
 	/* data: STA-addr SSID(hex) IEs(hex) */
 
-	cli = malloc(sizeof(*cli));
+	cli = wpa_zalloc(sizeof(*cli));
 	if (cli == NULL)
 		return;
 
-	memset(cli, 0, sizeof(*cli));
 	if (hwaddr_aton(data, cli->addr)) {
 		printf("test_socket: Invalid MAC address '%s' in ASSOC\n",
 		       data);
@@ -402,13 +401,12 @@ static int test_driver_init(struct hostapd_data *hapd)
 	struct test_driver_data *drv;
 	struct sockaddr_un addr;
 
-	drv = malloc(sizeof(struct test_driver_data));
+	drv = wpa_zalloc(sizeof(struct test_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for test driver data\n");
 		return -1;
 	}
 
-	memset(drv, 0, sizeof(*drv));
 	drv->ops = test_driver_ops;
 	drv->hapd = hapd;
 
