@@ -308,14 +308,13 @@ static int wired_send_eapol(void *priv, const u8 *addr,
 	int res;
 
 	len = sizeof(*hdr) + data_len;
-	hdr = malloc(len);
+	hdr = wpa_zalloc(len);
 	if (hdr == NULL) {
 		printf("malloc() failed for wired_send_eapol(len=%lu)\n",
 		       (unsigned long) len);
 		return -1;
 	}
 
-	memset(hdr, 0, len);
 	memcpy(hdr->dest, drv->use_pae_group_addr ? pae_group_addr : addr,
 	       ETH_ALEN);
 	memcpy(hdr->src, drv->hapd->own_addr, ETH_ALEN);
@@ -341,13 +340,12 @@ static int wired_driver_init(struct hostapd_data *hapd)
 {
 	struct wired_driver_data *drv;
 
-	drv = malloc(sizeof(struct wired_driver_data));
+	drv = wpa_zalloc(sizeof(struct wired_driver_data));
 	if (drv == NULL) {
 		printf("Could not allocate memory for wired driver data\n");
 		return -1;
 	}
 
-	memset(drv, 0, sizeof(*drv));
 	drv->ops = wired_driver_ops;
 	drv->hapd = hapd;
 	drv->use_pae_group_addr = hapd->conf->use_pae_group_addr;
