@@ -86,10 +86,9 @@ static void * eap_aka_init(struct eap_sm *sm)
 		return NULL;
 	}
 
-	data = malloc(sizeof(*data));
+	data = wpa_zalloc(sizeof(*data));
 	if (data == NULL)
-		return data;
-	memset(data, 0, sizeof(*data));
+		return NULL;
 	data->state = IDENTITY;
 	eap_aka_determine_identity(sm, data, 1);
 
