@@ -129,10 +129,9 @@ static void * eap_peap_init(struct eap_sm *sm)
 {
 	struct eap_peap_data *data;
 
-	data = malloc(sizeof(*data));
+	data = wpa_zalloc(sizeof(*data));
 	if (data == NULL)
-		return data;
-	memset(data, 0, sizeof(*data));
+		return NULL;
 	data->peap_version = EAP_PEAP_VERSION;
 	data->force_version = -1;
 	if (sm->user && sm->user->force_version >= 0) {
@@ -290,12 +289,10 @@ static u8 * eap_peap_build_phase2_term(struct eap_sm *sm,
 	struct eap_hdr *hdr;
 
 	req_len = sizeof(*hdr);
-	hdr = malloc(req_len);
-	if (hdr == NULL) {
+	hdr = wpa_zalloc(req_len);
+	if (hdr == NULL)
 		return NULL;
-	}
 
-	memset(hdr, 0, req_len);
 	hdr->code = success ? EAP_CODE_SUCCESS : EAP_CODE_FAILURE;
 	hdr->identifier = id;
 	hdr->length = htons(req_len);
