@@ -23,14 +23,14 @@ struct eloop_sock {
 	int sock;
 	void *eloop_data;
 	void *user_data;
-	void (*handler)(int sock, void *eloop_ctx, void *sock_ctx);
+	eloop_sock_handler handler;
 	WSAEVENT event;
 };
 
 struct eloop_event {
 	void *eloop_data;
 	void *user_data;
-	void (*handler)(void *eloop_data, void *user_data);
+	eloop_event_handler handler;
 	HANDLE event;
 };
 
@@ -38,14 +38,14 @@ struct eloop_timeout {
 	struct os_time time;
 	void *eloop_data;
 	void *user_data;
-	void (*handler)(void *eloop_ctx, void *sock_ctx);
+	eloop_timeout_handler handler;
 	struct eloop_timeout *next;
 };
 
 struct eloop_signal {
 	int sig;
 	void *user_data;
-	void (*handler)(int sig, void *eloop_ctx, void *signal_ctx);
+	eloop_signal_handler handler;
 	int signaled;
 };
 
@@ -108,9 +108,7 @@ static int eloop_prepare_handles(void)
 }
 
 
-int eloop_register_read_sock(int sock,
-			     void (*handler)(int sock, void *eloop_ctx,
-					     void *sock_ctx),
+int eloop_register_read_sock(int sock, eloop_sock_handler handler,
 			     void *eloop_data, void *user_data)
 {
 	WSAEVENT event;
@@ -182,7 +180,7 @@ void eloop_unregister_read_sock(int sock)
 
 
 int eloop_register_event(void *event, size_t event_size,
-			 void (*handler)(void *eloop_data, void *user_ctx),
+			 eloop_event_handler handler,
 			 void *eloop_data, void *user_data)
 {
 	struct eloop_event *tmp;
@@ -238,12 +236,12 @@ void eloop_unregister_event(void *event, size_t event_size)
 
 
 int eloop_register_timeout(unsigned int secs, unsigned int usecs,
-			   void (*handler)(void *eloop_ctx, void *timeout_ctx),
+			   eloop_timeout_handler handler,
 			   void *eloop_data, void *user_data)
 {
 	struct eloop_timeout *timeout, *tmp, *prev;
 
-	timeout = (struct eloop_timeout *) malloc(sizeof(*timeout));
+	timeout = malloc(sizeof(*timeout));
 	if (timeout == NULL)
 		return -1;
 	os_get_time(&timeout->time);
@@ -284,7 +282,7 @@ int eloop_register_timeout(unsigned int secs, unsigned int usecs,
 }
 
 
-int eloop_cancel_timeout(void (*handler)(void *eloop_ctx, void *sock_ctx),
+int eloop_cancel_timeout(eloop_timeout_handler handler,
 			 void *eloop_data, void *user_data)
 {
 	struct eloop_timeout *timeout, *prev, *next;
@@ -363,9 +361,7 @@ static void eloop_process_pending_signals(void)
 }
 
 
-int eloop_register_signal(int sig,
-			  void (*handler)(int sig, void *eloop_ctx,
-					  void *signal_ctx),
+int eloop_register_signal(int sig, eloop_signal_handler handler,
 			  void *user_data)
 {
 	struct eloop_signal *tmp;
@@ -407,8 +403,7 @@ static BOOL eloop_handle_console_ctrl(DWORD type)
 #endif /* _WIN32_WCE */
 
 
-int eloop_register_signal_terminate(void (*handler)(int sig, void *eloop_ctx,
-						    void *signal_ctx),
+int eloop_register_signal_terminate(eloop_signal_handler handler,
 				    void *user_data)
 {
 	if (eloop.term_event == NULL) {
@@ -436,8 +431,7 @@ int eloop_register_signal_terminate(void (*handler)(int sig, void *eloop_ctx,
 }
 
 
-int eloop_register_signal_reconfig(void (*handler)(int sig, void *eloop_ctx,
-						   void *signal_ctx),
+int eloop_register_signal_reconfig(eloop_signal_handler handler,
 				   void *user_data)
 {
 	/* TODO */
@@ -507,7 +501,6 @@ void eloop_run(void)
 
 		while (ret >= WAIT_OBJECT_0 &&
 		       ret < WAIT_OBJECT_0 + eloop.event_count) {
-			ResetEvent(eloop.events[ret].event);
 			eloop.events[ret].handler(
 				eloop.events[ret].eloop_data,
 				eloop.events[ret].user_data);
