@@ -142,6 +142,7 @@
 #define WLAN_EID_IBSS_PARAMS 6
 #define WLAN_EID_CHALLENGE 16
 #define WLAN_EID_RSN 48
+#define WLAN_EID_EXT_SUPP_RATES 50
 #define WLAN_EID_GENERIC 221
 
 
@@ -423,6 +424,7 @@ enum {
 	PRISM2_PARAM_PRIVACY_INVOKED = 37,
 	PRISM2_PARAM_TKIP_COUNTERMEASURES = 38,
 	PRISM2_PARAM_DROP_UNENCRYPTED = 39,
+	PRISM2_PARAM_SCAN_CHANNEL_MASK = 40,
 };
 
 enum { HOSTAP_ANTSEL_DO_NOT_TOUCH = 0, HOSTAP_ANTSEL_DIVERSITY = 1,
