@@ -1,6 +1,5 @@
 /*
  * hostapd / Initialization and configuration
- * Host AP kernel driver
  * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,12 +13,16 @@
  */
 
 #include "includes.h"
+#ifndef CONFIG_NATIVE_WINDOWS
 #include <syslog.h>
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 #include "eloop.h"
 #include "hostapd.h"
 #include "ieee802_1x.h"
 #include "ieee802_11.h"
+#include "beacon.h"
+#include "hw_features.h"
 #include "accounting.h"
 #include "eapol_sm.h"
 #include "iapp.h"
@@ -36,7 +39,11 @@
 #include "eap_sim_db.h"
 #include "eap.h"
 #include "version.h"
+#ifndef CONFIG_NATIVE_WINDOWS
 #include "hostap_common.h"
+#else /* CONFIG_NATIVE_WINDOWS */
+#define WLAN_REASON_MICHAEL_MIC_FAILURE 14
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 
 struct hapd_interfaces {
@@ -124,6 +131,7 @@ void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 		printf("\n");
 	}
 
+#ifndef CONFIG_NATIVE_WINDOWS
 	if ((conf_syslog & module) && level >= conf_syslog_level) {
 		int priority;
 		switch (level) {
@@ -148,6 +156,7 @@ void hostapd_logger(struct hostapd_data *hapd, const u8 *addr,
 		vsyslog(priority, format, ap);
 		va_end(ap);
 	}
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 	free(format);
 }
@@ -258,9 +267,11 @@ static void hostapd_wpa_auth_conf(struct hostapd_bss_config *conf,
 	wconf->wpa_gmk_rekey = conf->wpa_gmk_rekey;
 	wconf->rsn_preauth = conf->rsn_preauth;
 	wconf->stakey = conf->stakey;
+	wconf->eapol_version = conf->eapol_version;
 }
 
 
+#ifndef CONFIG_NATIVE_WINDOWS
 static void handle_reload(int sig, void *eloop_ctx, void *signal_ctx)
 {
 	struct hapd_interfaces *hapds = (struct hapd_interfaces *) eloop_ctx;
@@ -281,7 +292,7 @@ static void handle_reload(int sig, void *eloop_ctx, void *signal_ctx)
 		/* TODO: update dynamic data based on changed configuration
 		 * items (e.g., open/close sockets, remove stations added to
 		 * deny list, etc.) */
-		radius_client_flush(hapd->radius);
+		radius_client_flush(hapd->radius, 0);
 		hostapd_config_free(hapd->iconf);
 
 		hostapd_wpa_auth_conf(&newconf->bss[0], &wpa_auth_conf);
@@ -338,14 +349,9 @@ static void hostapd_dump_state(struct hostapd_data *hapd)
 			sta->listen_interval);
 
 		fprintf(f, "  supported_rates=");
-		for (i = 0; i < sizeof(sta->supported_rates); i++)
-			if (sta->supported_rates[i] != 0)
-				fprintf(f, "%02x ", sta->supported_rates[i]);
-		fprintf(f, "%s%s%s%s\n",
-			(sta->tx_supp_rates & WLAN_RATE_1M ? "[1M]" : ""),
-			(sta->tx_supp_rates & WLAN_RATE_2M ? "[2M]" : ""),
-			(sta->tx_supp_rates & WLAN_RATE_5M5 ? "[5.5M]" : ""),
-			(sta->tx_supp_rates & WLAN_RATE_11M ? "[11M]" : ""));
+		for (i = 0; i < sta->supported_rates_len; i++)
+			fprintf(f, "%02x ", sta->supported_rates[i]);
+		fprintf(f, "\n");
 
 		fprintf(f,
 			"  timeout_next=%s\n",
@@ -393,6 +399,7 @@ static void handle_dump_state(int sig, void *eloop_ctx, void *signal_ctx)
 	}
 #endif /* HOSTAPD_DUMP_STATE */
 }
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 
 static void hostapd_cleanup(struct hostapd_data *hapd)
@@ -445,6 +452,8 @@ static void hostapd_cleanup(struct hostapd_data *hapd)
 
 static void hostapd_cleanup_iface(struct hostapd_iface *iface)
 {
+	hostapd_free_hw_features(iface->hw_features, iface->num_hw_features);
+	free(iface->current_rates);
 	hostapd_config_free(iface->conf);
 
 	free(iface->config_fname);
@@ -457,12 +466,12 @@ static int hostapd_flush_old_stations(struct hostapd_data *hapd)
 {
 	int ret = 0;
 
-	printf("Flushing old station entries\n");
+	wpa_printf(MSG_DEBUG, "Flushing old station entries");
 	if (hostapd_flush(hapd)) {
 		printf("Could not connect to kernel driver.\n");
 		ret = -1;
 	}
-	printf("Deauthenticate all stations\n");
+	wpa_printf(MSG_DEBUG, "Deauthenticate all stations");
 	hostapd_deauth_all_stas(hapd);
 
 	return ret;
@@ -657,7 +666,7 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 	struct hostapd_data *hapd = iface->bss[0];
 	struct hostapd_bss_config *conf = hapd->conf;
 	u8 ssid[HOSTAPD_MAX_SSID_LEN + 1];
-	int ssid_len, set_ssid;
+	int ssid_len, set_ssid, freq;
 	int ret = 0;
 
 	if (hostapd_driver_init(hapd)) {
@@ -804,6 +813,8 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 	if (hostapd_wireless_event_init(hapd) < 0)
 		return -1;
 
+	ieee802_11_set_beacon(hapd);
+
 	if (hostapd_flush_old_stations(hapd))
 		return -1;
 
@@ -812,6 +823,25 @@ static int hostapd_setup_interface(struct hostapd_iface *iface)
 		ret = -1;
 	}
 
+	if (hostapd_get_hw_features(iface)) {
+		/* Not all drivers support this yet, so continue without hw
+		 * feature data. */
+	} else if (hostapd_select_hw_mode(iface)) {
+		printf("Could not select hw_mode and channel.\n");
+	}
+
+	if (hapd->iconf->channel) {
+		freq = hostapd_hw_get_freq(hapd, hapd->iconf->channel);
+		printf("Mode: %s  Channel: %d  Frequency: %d MHz\n",
+		       hostapd_hw_mode_txt(hapd->iconf->hw_mode),
+		       hapd->iconf->channel, freq);
+
+		if (hostapd_set_freq(hapd, hapd->iconf->hw_mode, freq)) {
+			printf("Could not set channel for kernel driver\n");
+			return -1;
+		}
+	}
+
 	return ret;
 }
 
@@ -932,10 +962,9 @@ hostapd_alloc_bss_data(struct hostapd_iface *hapd_iface,
 {
 	struct hostapd_data *hapd;
 
-	hapd = malloc(sizeof(*hapd));
+	hapd = wpa_zalloc(sizeof(*hapd));
 	if (hapd == NULL)
 		return NULL;
-	memset(hapd, 0, sizeof(*hapd));
 
 	hapd->iconf = conf;
 	hapd->conf = bss;
@@ -997,7 +1026,9 @@ hostapd_alloc_bss_data(struct hostapd_iface *hapd_iface,
 
 	return hapd;
 
+#if defined(EAP_TLS_FUNCS) || defined(EAP_SERVER)
 fail:
+#endif
 	/* TODO: cleanup allocated resources(?) */
 	free(hapd);
 	return NULL;
@@ -1011,10 +1042,9 @@ static struct hostapd_iface * hostapd_init(const char *config_file)
 	struct hostapd_data *hapd;
 	int i;
 
-	hapd_iface = malloc(sizeof(*hapd_iface));
+	hapd_iface = wpa_zalloc(sizeof(*hapd_iface));
 	if (hapd_iface == NULL)
 		goto fail;
-	memset(hapd_iface, 0, sizeof(*hapd_iface));
 
 	hapd_iface->config_fname = strdup(config_file);
 	if (hapd_iface->config_fname == NULL)
@@ -1026,12 +1056,10 @@ static struct hostapd_iface * hostapd_init(const char *config_file)
 	hapd_iface->conf = conf;
 
 	hapd_iface->num_bss = conf->num_bss;
-	hapd_iface->bss = malloc(conf->num_bss *
-				 sizeof(struct hostapd_data *));
+	hapd_iface->bss = wpa_zalloc(conf->num_bss *
+				     sizeof(struct hostapd_data *));
 	if (hapd_iface->bss == NULL)
 		goto fail;
-	memset(hapd_iface->bss, 0, conf->num_bss *
-	       sizeof(struct hostapd_data *));
 
 	for (i = 0; i < conf->num_bss; i++) {
 		hapd = hapd_iface->bss[i] =
@@ -1131,10 +1159,11 @@ int main(int argc, char *argv[])
 		return -1;
 	}
 
+#ifndef CONFIG_NATIVE_WINDOWS
 	eloop_register_signal(SIGHUP, handle_reload, NULL);
-	eloop_register_signal(SIGINT, handle_term, NULL);
-	eloop_register_signal(SIGTERM, handle_term, NULL);
 	eloop_register_signal(SIGUSR1, handle_dump_state, NULL);
+#endif /* CONFIG_NATIVE_WINDOWS */
+	eloop_register_signal_terminate(handle_term, NULL);
 
 	for (i = 0; i < interfaces.count; i++) {
 		printf("Configuration file: %s\n", argv[optind + i]);
@@ -1152,12 +1181,14 @@ int main(int argc, char *argv[])
 			goto out;
 	}
 
-	if (daemonize && daemon(0, 0)) {
+	if (daemonize && os_daemonize(NULL)) {
 		perror("daemon");
 		goto out;
 	}
 
+#ifndef CONFIG_NATIVE_WINDOWS
 	openlog("hostapd", 0, LOG_DAEMON);
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 	eloop_run();
 
@@ -1191,11 +1222,15 @@ int main(int argc, char *argv[])
 
 	eloop_destroy();
 
+#ifndef CONFIG_NATIVE_WINDOWS
 	closelog();
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 	eap_server_unregister_methods();
 
 	driver_unregister_all();
 
+	os_daemonize_terminate(NULL);
+
 	return ret;
 }
