@@ -13,11 +13,16 @@
  */
 
 #include "includes.h"
+
+#ifndef CONFIG_NATIVE_WINDOWS
+
 #include <net/if.h>
 
 #include "eloop.h"
 #include "hostapd.h"
 #include "ieee802_11.h"
+#include "beacon.h"
+#include "hw_features.h"
 #include "radius.h"
 #include "radius_client.h"
 #include "ieee802_11_auth.h"
@@ -31,29 +36,101 @@
 #include "hostap_common.h"
 
 
-static u8 * hostapd_eid_supp_rates(struct hostapd_data *hapd, u8 *eid)
+u8 * hostapd_eid_supp_rates(struct hostapd_data *hapd, u8 *eid)
 {
 	u8 *pos = eid;
+	int i, num, count;
+
+	if (hapd->iface->current_rates == NULL)
+		return eid;
 
 	*pos++ = WLAN_EID_SUPP_RATES;
-	*pos++ = 4; /* len */
-	*pos++ = 0x82; /* 1 Mbps, base set */
-	*pos++ = 0x84; /* 2 Mbps, base set */
-	*pos++ = 0x0b; /* 5.5 Mbps */
-	*pos++ = 0x16; /* 11 Mbps */
+	num = hapd->iface->num_rates;
+	if (num > 8) {
+		/* rest of the rates are encoded in Extended supported
+		 * rates element */
+		num = 8;
+	}
+
+	*pos++ = num;
+	count = 0;
+	for (i = 0, count = 0; i < hapd->iface->num_rates && count < num;
+	     i++) {
+		count++;
+		*pos = hapd->iface->current_rates[i].rate / 5;
+		if (hapd->iface->current_rates[i].flags & HOSTAPD_RATE_BASIC)
+			*pos |= 0x80;
+		pos++;
+	}
 
 	return pos;
 }
 
 
-static u16 hostapd_own_capab_info(struct hostapd_data *hapd)
+u8 * hostapd_eid_ext_supp_rates(struct hostapd_data *hapd, u8 *eid)
+{
+	u8 *pos = eid;
+	int i, num, count;
+
+	if (hapd->iface->current_rates == NULL)
+		return eid;
+
+	num = hapd->iface->num_rates;
+	if (num <= 8)
+		return eid;
+	num -= 8;
+
+	*pos++ = WLAN_EID_EXT_SUPP_RATES;
+	*pos++ = num;
+	count = 0;
+	for (i = 0, count = 0; i < hapd->iface->num_rates && count < num + 8;
+	     i++) {
+		count++;
+		if (count <= 8)
+			continue; /* already in SuppRates IE */
+		*pos = hapd->iface->current_rates[i].rate / 5;
+		if (hapd->iface->current_rates[i].flags & HOSTAPD_RATE_BASIC)
+			*pos |= 0x80;
+		pos++;
+	}
+
+	return pos;
+}
+
+
+u16 hostapd_own_capab_info(struct hostapd_data *hapd, struct sta_info *sta,
+			   int probe)
 {
 	int capab = WLAN_CAPABILITY_ESS;
-	if (hapd->conf->wpa ||
-	    (hapd->conf->ieee802_1x &&
-	     (hapd->conf->default_wep_key_len ||
-	      hapd->conf->individual_wep_key_len)))
+	int privacy;
+
+	privacy = hapd->conf->ssid.wep.keys_set;
+
+	if (hapd->conf->ieee802_1x &&
+	    (hapd->conf->default_wep_key_len ||
+	     hapd->conf->individual_wep_key_len))
+		privacy = 1;
+
+	if (hapd->conf->wpa)
+		privacy = 1;
+
+	if (sta) {
+		int policy, def_klen;
+		if (probe && sta->ssid_probe) {
+			policy = sta->ssid_probe->security_policy;
+			def_klen = sta->ssid_probe->wep.default_len;
+		} else {
+			policy = sta->ssid->security_policy;
+			def_klen = sta->ssid->wep.default_len;
+		}
+		privacy = policy != SECURITY_PLAINTEXT;
+		if (policy == SECURITY_IEEE_802_1X && def_klen == 0)
+			privacy = 0;
+	}
+
+	if (privacy)
 		capab |= WLAN_CAPABILITY_PRIVACY;
+
 	return capab;
 }
 
@@ -123,6 +200,10 @@ ParseRes ieee802_11_parse_elems(struct hostapd_data *hapd, u8 *start,
 			elems->challenge = pos;
 			elems->challenge_len = elen;
 			break;
+		case WLAN_EID_EXT_SUPP_RATES:
+			elems->ext_supp_rates = pos;
+			elems->ext_supp_rates_len = elen;
+			break;
 		case WLAN_EID_GENERIC:
 			if (elen > 4 && memcmp(pos, WPA_OUI_TYPE, 4) == 0) {
 				elems->wpa_ie = pos;
@@ -169,7 +250,7 @@ ParseRes ieee802_11_parse_elems(struct hostapd_data *hapd, u8 *start,
 }
 
 
-static void ieee802_11_print_ssid(const u8 *ssid, u8 len)
+void ieee802_11_print_ssid(const u8 *ssid, u8 len)
 {
 	int i;
 	for (i = 0; i < len; i++) {
@@ -253,6 +334,7 @@ static void ieee802_11_sta_associate(void *eloop_ctx, void *timeout_ctx)
 	p += hapd->assoc_ap_ssid_len;
 
 	p = hostapd_eid_supp_rates(hapd, p);
+	p = hostapd_eid_ext_supp_rates(hapd, p);
 
 	if (hostapd_send_mgmt_frame(hapd, mgmt, p - (u8 *) mgmt, 0) < 0)
 		perror("ieee802_11_sta_associate: send");
@@ -276,10 +358,9 @@ static u16 auth_shared_key(struct hostapd_data *hapd, struct sta_info *sta,
 			u8 key[8];
 			time_t now;
 			int r;
-			sta->challenge = malloc(WLAN_AUTH_CHALLENGE_LEN);
-			if (!sta->challenge)
+			sta->challenge = wpa_zalloc(WLAN_AUTH_CHALLENGE_LEN);
+			if (sta->challenge == NULL)
 				return WLAN_STATUS_UNSPECIFIED_FAILURE;
-			memset(sta->challenge, 0, WLAN_AUTH_CHALLENGE_LEN);
 
 			now = time(NULL);
 			r = random();
@@ -521,7 +602,7 @@ static void handle_assoc(struct hostapd_data *hapd,
 	u16 resp = WLAN_STATUS_SUCCESS;
 	u8 *pos, *wpa_ie;
 	size_t wpa_ie_len;
-	int send_deauth = 0, send_len, left, i;
+	int send_deauth = 0, send_len, left;
 	struct sta_info *sta;
 	struct ieee802_11_elems elems;
 
@@ -595,32 +676,44 @@ static void handle_assoc(struct hostapd_data *hapd,
 		goto fail;
 	}
 
-	if (elems.supp_rates) {
-		if (elems.supp_rates_len > sizeof(sta->supported_rates)) {
-			printf("STA " MACSTR ": Invalid supported rates "
-			       "element length %d\n", MAC2STR(sta->addr),
+	if (!elems.supp_rates) {
+		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
+			       HOSTAPD_LEVEL_DEBUG,
+			       "No supported rates element in AssocReq");
+		resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
+		goto fail;
+	}
+
+	if (elems.supp_rates_len > sizeof(sta->supported_rates)) {
+		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
+			       HOSTAPD_LEVEL_DEBUG,
+			       "Invalid supported rates element length %d",
 			       elems.supp_rates_len);
+		resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
+		goto fail;
+	}
+
+	memset(sta->supported_rates, 0, sizeof(sta->supported_rates));
+	memcpy(sta->supported_rates, elems.supp_rates, elems.supp_rates_len);
+	sta->supported_rates_len = elems.supp_rates_len;
+
+	if (elems.ext_supp_rates) {
+		if (elems.supp_rates_len + elems.ext_supp_rates_len >
+		    sizeof(sta->supported_rates)) {
+			hostapd_logger(hapd, mgmt->sa,
+				       HOSTAPD_MODULE_IEEE80211,
+				       HOSTAPD_LEVEL_DEBUG,
+				       "Invalid supported rates element length"
+				       " %d+%d", elems.supp_rates_len,
+				       elems.ext_supp_rates_len);
 			resp = WLAN_STATUS_UNSPECIFIED_FAILURE;
 			goto fail;
 		}
 
-		memset(sta->supported_rates, 0, sizeof(sta->supported_rates));
-		memcpy(sta->supported_rates, elems.supp_rates,
-		       elems.supp_rates_len);
-
-		sta->tx_supp_rates = 0;
-		for (i = 0; i < elems.supp_rates_len; i++) {
-			if ((sta->supported_rates[i] & 0x7f) == 2)
-				sta->tx_supp_rates |= WLAN_RATE_1M;
-			if ((sta->supported_rates[i] & 0x7f) == 4)
-				sta->tx_supp_rates |= WLAN_RATE_2M;
-			if ((sta->supported_rates[i] & 0x7f) == 11)
-				sta->tx_supp_rates |= WLAN_RATE_5M5;
-			if ((sta->supported_rates[i] & 0x7f) == 22)
-				sta->tx_supp_rates |= WLAN_RATE_11M;
-		}
-	} else
-		sta->tx_supp_rates = 0xff;
+		memcpy(sta->supported_rates + elems.supp_rates_len,
+		       elems.ext_supp_rates, elems.ext_supp_rates_len);
+		sta->supported_rates_len += elems.ext_supp_rates_len;
+	}
 
 	if ((hapd->conf->wpa & HOSTAPD_WPA_VERSION_WPA2) && elems.rsn_ie) {
 		wpa_ie = elems.rsn_ie;
@@ -725,12 +818,14 @@ static void handle_assoc(struct hostapd_data *hapd,
 		u8 *p;
 		send_len += sizeof(mgmt->u.assoc_resp);
 		mgmt->u.assoc_resp.capab_info =
-			host_to_le16(hostapd_own_capab_info(hapd));
+			host_to_le16(hostapd_own_capab_info(hapd, sta, 0));
 		mgmt->u.assoc_resp.status_code = host_to_le16(resp);
 		mgmt->u.assoc_resp.aid = host_to_le16((sta ? sta->aid : 0)
 						      | BIT(14) | BIT(15));
 		/* Supported rates */
 		p = hostapd_eid_supp_rates(hapd, mgmt->u.assoc_resp.variable);
+		/* Extended supported rates */
+		p = hostapd_eid_ext_supp_rates(hapd, p);
 		send_len += p - mgmt->u.assoc_resp.variable;
 
 		/* Request TX callback */
@@ -938,9 +1033,23 @@ static void handle_beacon(struct hostapd_data *hapd,
 }
 
 
+/**
+ * ieee802_11_mgmt - process incoming IEEE 802.11 management frames
+ * @hapd: hostapd BSS data structure (the BSS to which the management frame was
+ * sent to)
+ * @buf: management frame data (starting from IEEE 802.11 header)
+ * @len: length of frame data in octets
+ * @stype: management frame subtype from frame control field
+ *
+ * Process all incoming IEEE 802.11 management frames. This will be called for
+ * each frame received from the kernel driver through wlan#ap interface. In
+ * addition, it can be called to re-inserted pending frames (e.g., when using
+ * external RADIUS server as an MAC ACL).
+ */
 void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
 {
 	struct ieee80211_mgmt *mgmt = (struct ieee80211_mgmt *) buf;
+	int broadcast;
 
 	if (stype == WLAN_FC_STYPE_BEACON) {
 		HOSTAPD_DEBUG(HOSTAPD_DEBUG_EXCESSIVE, "mgmt::beacon\n");
@@ -948,7 +1057,11 @@ void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
 		return;
 	}
 
-	if (memcmp(mgmt->bssid, hapd->own_addr, ETH_ALEN) != 0 &&
+	broadcast = mgmt->bssid[0] == 0xff && mgmt->bssid[1] == 0xff &&
+		mgmt->bssid[2] == 0xff && mgmt->bssid[3] == 0xff &&
+		mgmt->bssid[4] == 0xff && mgmt->bssid[5] == 0xff;
+
+	if (!broadcast && memcmp(mgmt->bssid, hapd->own_addr, ETH_ALEN) != 0 &&
 	    (hapd->assoc_ap_state == DO_NOT_ASSOC ||
 	     memcmp(mgmt->bssid, hapd->conf->assoc_ap_addr, ETH_ALEN) != 0)) {
 		printf("MGMT: BSSID=" MACSTR " not our address\n",
@@ -958,13 +1071,16 @@ void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
 
 
 	if (stype == WLAN_FC_STYPE_PROBE_REQ) {
-		printf("mgmt::probe_req\n");
+		HOSTAPD_DEBUG(HOSTAPD_DEBUG_MSGDUMPS, "mgmt::probe_req\n");
+		handle_probe_req(hapd, mgmt, len);
 		return;
 	}
 
 	if (memcmp(mgmt->da, hapd->own_addr, ETH_ALEN) != 0) {
-		printf("MGMT: DA=" MACSTR " not our address\n",
-		       MAC2STR(mgmt->da));
+		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
+			       HOSTAPD_LEVEL_DEBUG,
+			       "MGMT: DA=" MACSTR " not our address",
+			       MAC2STR(mgmt->da));
 		return;
 	}
 
@@ -994,7 +1110,9 @@ void ieee802_11_mgmt(struct hostapd_data *hapd, u8 *buf, size_t len, u16 stype)
 		handle_deauth(hapd, mgmt, len);
 		break;
 	default:
-		printf("unknown mgmt frame subtype %d\n", stype);
+		hostapd_logger(hapd, mgmt->sa, HOSTAPD_MODULE_IEEE80211,
+			       HOSTAPD_LEVEL_DEBUG,
+			       "unknown mgmt frame subtype %d", stype);
 		break;
 	}
 }
@@ -1094,8 +1212,11 @@ static void handle_assoc_cb(struct hostapd_data *hapd,
 	sta->flags |= WLAN_STA_ASSOC;
 
 	if (hostapd_sta_add(hapd->conf->iface, hapd, sta->addr, sta->aid,
-			    sta->capability, sta->tx_supp_rates)) {
-		printf("Could not add station to kernel driver.\n");
+			    sta->capability, sta->supported_rates,
+			    sta->supported_rates_len, 0)) {
+		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE80211,
+			       HOSTAPD_LEVEL_NOTICE,
+			       "Could not add STA to kernel driver");
 	}
 
 	wpa_auth_sm_event(sta->wpa_sm, WPA_ASSOC);
@@ -1221,3 +1342,5 @@ int ieee802_11_get_mib_sta(struct hostapd_data *hapd, struct sta_info *sta,
 	/* TODO */
 	return 0;
 }
+
+#endif /* CONFIG_NATIVE_WINDOWS */
