@@ -14,6 +14,8 @@
 
 #include "includes.h"
 
+#ifndef CONFIG_NATIVE_WINDOWS
+
 #include "hostapd.h"
 #include "ieee802_11.h"
 #include "ieee802_11_auth.h"
@@ -236,12 +238,11 @@ int hostapd_allowed_address(struct hostapd_data *hapd, const u8 *addr,
 			return HOSTAPD_ACL_REJECT;
 
 		/* No entry in the cache - query external RADIUS server */
-		query = malloc(sizeof(*query));
+		query = wpa_zalloc(sizeof(*query));
 		if (query == NULL) {
 			printf("malloc for query data failed\n");
 			return HOSTAPD_ACL_REJECT;
 		}
-		memset(query, 0, sizeof(*query));
 		time(&query->timestamp);
 		memcpy(query->addr, addr, ETH_ALEN);
 		if (hostapd_radius_acl_query(hapd, addr, query)) {
@@ -381,12 +382,11 @@ hostapd_acl_recv_radius(struct radius_msg *msg, struct radius_msg *req,
 	}
 
 	/* Insert Accept/Reject info into ACL cache */
-	cache = malloc(sizeof(*cache));
+	cache = wpa_zalloc(sizeof(*cache));
 	if (cache == NULL) {
 		printf("Failed to add ACL cache entry\n");
 		goto done;
 	}
-	memset(cache, 0, sizeof(*cache));
 	time(&cache->timestamp);
 	memcpy(cache->addr, query->addr, sizeof(cache->addr));
 	if (msg->hdr->code == RADIUS_CODE_ACCESS_ACCEPT) {
@@ -457,3 +457,5 @@ void hostapd_acl_deinit(struct hostapd_data *hapd)
 		hostapd_acl_query_free(prev);
 	}
 }
+
+#endif /* CONFIG_NATIVE_WINDOWS */
