@@ -1,6 +1,6 @@
 /*
  * hostapd / IEEE 802.1X Authenticator
- * Copyright (c) 2002-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2002-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -45,14 +45,13 @@ static void ieee802_1x_send(struct hostapd_data *hapd, struct sta_info *sta,
 	int encrypt = 0;
 
 	len = sizeof(*xhdr) + datalen;
-	buf = malloc(len);
+	buf = wpa_zalloc(len);
 	if (buf == NULL) {
 		printf("malloc() failed for ieee802_1x_send(len=%lu)\n",
 		       (unsigned long) len);
 		return;
 	}
 
-	memset(buf, 0, len);
 #if 0
 	/* TODO:
 	 * According to IEEE 802.1aa/D4 EAPOL-Key should be sent before any
@@ -66,7 +65,7 @@ static void ieee802_1x_send(struct hostapd_data *hapd, struct sta_info *sta,
 #endif
 
 	xhdr = (struct ieee802_1x_hdr *) buf;
-	xhdr->version = EAPOL_VERSION;
+	xhdr->version = hapd->conf->eapol_version;
 	xhdr->type = type;
 	xhdr->length = htons(datalen);
 
@@ -88,20 +87,30 @@ static void ieee802_1x_send(struct hostapd_data *hapd, struct sta_info *sta,
 void ieee802_1x_set_sta_authorized(struct hostapd_data *hapd,
 				   struct sta_info *sta, int authorized)
 {
+	int res;
+
 	if (sta->flags & WLAN_STA_PREAUTH)
 		return;
 
 	if (authorized) {
 		sta->flags |= WLAN_STA_AUTHORIZED;
+		res = hostapd_sta_set_flags(hapd, sta->addr,
+					    WLAN_STA_AUTHORIZED, ~0);
 		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE8021X,
 			       HOSTAPD_LEVEL_DEBUG, "authorizing port");
 	} else {
 		sta->flags &= ~WLAN_STA_AUTHORIZED;
+		res = hostapd_sta_set_flags(hapd, sta->addr,
+					    0, ~WLAN_STA_AUTHORIZED);
 		hostapd_logger(hapd, sta->addr, HOSTAPD_MODULE_IEEE8021X,
 			       HOSTAPD_LEVEL_DEBUG, "unauthorizing port");
 	}
 
-	hostapd_set_sta_authorized(hapd, sta->addr, authorized);
+	if (res && errno != ENOENT) {
+		printf("Could not set station " MACSTR " flags for kernel "
+		       "driver (errno=%d).\n", MAC2STR(sta->addr), errno);
+	}
+
 	if (authorized)
 		accounting_sta_start(hapd, sta);
 }
@@ -143,14 +152,12 @@ void ieee802_1x_request_identity(struct hostapd_data *hapd,
 
 	tlen = sizeof(*eap) + 1 + hapd->conf->eap_req_id_text_len;
 
-	buf = malloc(tlen);
+	buf = wpa_zalloc(tlen);
 	if (buf == NULL) {
 		printf("Could not allocate memory for identity request\n");
 		return;
 	}
 
-	memset(buf, 0, tlen);
-
 	eap = (struct eap_hdr *) buf;
 	eap->code = EAP_CODE_REQUEST;
 	eap->identifier = ++sm->currentId;
@@ -255,11 +262,10 @@ static void ieee802_1x_tx_key_one(struct hostapd_data *hapd,
 		return;
 
 	len = sizeof(*key) + key_len;
-	buf = malloc(sizeof(*hdr) + len);
+	buf = wpa_zalloc(sizeof(*hdr) + len);
 	if (buf == NULL)
 		return;
 
-	memset(buf, 0, sizeof(*hdr) + len);
 	hdr = (struct ieee802_1x_hdr *) buf;
 	key = (struct ieee802_1x_eapol_key *) (hdr + 1);
 	key->type = EAPOL_KEY_TYPE_RC4;
@@ -299,7 +305,7 @@ static void ieee802_1x_tx_key_one(struct hostapd_data *hapd,
 
 	/* This header is needed here for HMAC-MD5, but it will be regenerated
 	 * in ieee802_1x_send() */
-	hdr->version = EAPOL_VERSION;
+	hdr->version = hapd->conf->eapol_version;
 	hdr->type = IEEE802_1X_TYPE_EAPOL_KEY;
 	hdr->length = htons(len);
 	hmac_md5(sm->eapol_key_sign, sm->eapol_key_sign_len,
@@ -847,11 +853,10 @@ int ieee802_1x_copy_radius_class(struct radius_class_data *dst,
 	if (src->attr == NULL)
 		return 0;
 
-	dst->attr = malloc(src->count * sizeof(struct radius_attr_data));
+	dst->attr = wpa_zalloc(src->count * sizeof(struct radius_attr_data));
 	if (dst->attr == NULL)
 		return -1;
 
-	memset(dst->attr, 0, src->count * sizeof(struct radius_attr_data));
 	dst->count = 0;
 
 	for (i = 0; i < src->count; i++) {
@@ -1036,12 +1041,11 @@ static void ieee802_1x_store_radius_class(struct hostapd_data *hapd,
 	if (count <= 0)
 		return;
 
-	nclass = malloc(count * sizeof(struct radius_attr_data));
+	nclass = wpa_zalloc(count * sizeof(struct radius_attr_data));
 	if (nclass == NULL)
 		return;
 
 	nclass_count = 0;
-	memset(nclass, 0, count * sizeof(struct radius_attr_data));
 
 	class = NULL;
 	for (i = 0; i < count; i++) {
