@@ -365,7 +365,7 @@ void pbkdf2_sha1(const char *passphrase, const char *ssid, size_t ssid_len,
 }
 
 
-#ifndef EAP_TLS_FUNCS
+#if !defined(EAP_TLS_FUNCS) || defined(EAP_TLS_NONE)
 
 typedef struct {
 	u32 state[5];
@@ -676,7 +676,7 @@ static void SHA1Final(unsigned char digest[20], SHA1_CTX* context)
 
 /* ===== end - public domain SHA1 implementation ===== */
 
-#endif /* EAP_TLS_FUNCS */
+#endif /* !EAP_TLS_FUNCS || EAP_TLS_NONE */
 
 
 #ifdef TEST_MAIN
