@@ -26,7 +26,12 @@
 #include "preauth.h"
 #include "radius_client.h"
 #include "driver.h"
+#ifndef CONFIG_NATIVE_WINDOWS
 #include "hostap_common.h"
+#else /* CONFIG_NATIVE_WINDOWS */
+#define WLAN_REASON_PREV_AUTH_NOT_VALID 2
+#define WLAN_REASON_DISASSOC_DUE_TO_INACTIVITY 4
+#endif /* CONFIG_NATIVE_WINDOWS */
 
 
 int ap_for_each_sta(struct hostapd_data *hapd,
@@ -212,6 +217,7 @@ void ap_handle_timer(void *eloop_ctx, void *timeout_ctx)
 			      "  Polling STA with data frame\n");
 		sta->flags |= WLAN_STA_PENDING_POLL;
 
+#ifndef CONFIG_NATIVE_WINDOWS
 		/* FIX: WLAN_FC_STYPE_NULLFUNC would be more appropriate, but
 		 * it is apparently not retried so TX Exc events are not
 		 * received for it */
@@ -226,6 +232,7 @@ void ap_handle_timer(void *eloop_ctx, void *timeout_ctx)
 
 		if (hostapd_send_mgmt_frame(hapd, &hdr, sizeof(hdr), 0) < 0)
 			perror("ap_handle_timer: send");
+#endif /* CONFIG_NATIVE_WINDOWS */
 	} else if (sta->timeout_next != STA_REMOVE) {
 		int deauth = sta->timeout_next == STA_DEAUTH;
 
@@ -330,12 +337,11 @@ struct sta_info * ap_sta_add(struct hostapd_data *hapd, const u8 *addr)
 		return NULL;
 	}
 
-	sta = (struct sta_info *) malloc(sizeof(struct sta_info));
+	sta = wpa_zalloc(sizeof(struct sta_info));
 	if (sta == NULL) {
 		printf("  malloc failed\n");
 		return NULL;
 	}
-	memset(sta, 0, sizeof(struct sta_info));
 	sta->acct_interim_interval = hapd->conf->radius->acct_interim_interval;
 
 	/* initialize STA info data */
