@@ -175,10 +175,9 @@ void * tls_init(const struct tls_config *conf)
 	int i;
 #endif /* GNUTLS_INTERNAL_STRUCTURE_HACK */
 
-	global = malloc(sizeof(*global));
+	global = wpa_zalloc(sizeof(*global));
 	if (global == NULL)
 		return NULL;
-	memset(global, 0, sizeof(*global));
 
 	if (tls_gnutls_ref_count == 0 && gnutls_global_init() < 0) {
 		free(global);
@@ -328,10 +327,9 @@ struct tls_connection * tls_connection_init(void *ssl_ctx)
 	struct tls_connection *conn;
 	int ret;
 
-	conn = malloc(sizeof(*conn));
+	conn = wpa_zalloc(sizeof(*conn));
 	if (conn == NULL)
 		return NULL;
-	memset(conn, 0, sizeof(*conn));
 
 	if (tls_gnutls_init_session(global, conn)) {
 		free(conn);
@@ -1151,14 +1149,12 @@ int tls_connection_resumed(void *ssl_ctx, struct tls_connection *conn)
 }
 
 
-#ifdef EAP_FAST
 int tls_connection_set_master_key(void *ssl_ctx, struct tls_connection *conn,
 				  const u8 *key, size_t key_len)
 {
 	/* TODO */
 	return -1;
 }
-#endif /* EAP_FAST */
 
 
 int tls_connection_set_anon_dh(void *ssl_ctx, struct tls_connection *conn)
@@ -1185,7 +1181,6 @@ int tls_connection_enable_workaround(void *ssl_ctx,
 }
 
 
-#ifdef EAP_FAST
 int tls_connection_client_hello_ext(void *ssl_ctx, struct tls_connection *conn,
 				    int ext_type, const u8 *data,
 				    size_t data_len)
@@ -1193,7 +1188,6 @@ int tls_connection_client_hello_ext(void *ssl_ctx, struct tls_connection *conn,
 	/* TODO */
 	return -1;
 }
-#endif /* EAP_FAST */
 
 
 int tls_connection_get_failed(void *ssl_ctx, struct tls_connection *conn)
