@@ -1,6 +1,6 @@
 /*
  * hostapd - IEEE 802.11i-2004 / WPA Authenticator
- * Copyright (c) 2004-2005, Jouni Malinen <jkmaline@cc.hut.fi>
+ * Copyright (c) 2004-2006, Jouni Malinen <jkmaline@cc.hut.fi>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License version 2 as
@@ -14,6 +14,8 @@
 
 #include "includes.h"
 
+#ifndef CONFIG_NATIVE_WINDOWS
+
 #include "hostapd.h"
 #include "eapol_sm.h"
 #include "wpa.h"
@@ -696,10 +698,9 @@ struct wpa_authenticator * wpa_init(const u8 *addr,
 	u8 rkey[32];
 	u8 buf[ETH_ALEN + 8];
 
-	wpa_auth = malloc(sizeof(struct wpa_authenticator));
+	wpa_auth = wpa_zalloc(sizeof(struct wpa_authenticator));
 	if (wpa_auth == NULL)
 		return NULL;
-	memset(wpa_auth, 0, sizeof(*wpa_auth));
 	memcpy(wpa_auth->addr, addr, ETH_ALEN);
 	memcpy(&wpa_auth->conf, conf, sizeof(*conf));
 	memcpy(&wpa_auth->cb, cb, sizeof(*cb));
@@ -1334,10 +1335,9 @@ wpa_auth_sta_init(struct wpa_authenticator *wpa_auth, const u8 *addr)
 {
 	struct wpa_state_machine *sm;
 
-	sm = malloc(sizeof(struct wpa_state_machine));
+	sm = wpa_zalloc(sizeof(struct wpa_state_machine));
 	if (sm == NULL)
 		return NULL;
-	memset(sm, 0, sizeof(struct wpa_state_machine));
 	memcpy(sm->addr, addr, ETH_ALEN);
 
 	sm->wpa_auth = wpa_auth;
@@ -1556,10 +1556,9 @@ static int wpa_stakey_initiate(struct wpa_authenticator *wpa_auth,
 		return -1;
 	}
 
-	neg = malloc(sizeof(*neg));
+	neg = wpa_zalloc(sizeof(*neg));
 	if (neg == NULL)
 		return -1;
-	memset(neg, 0, sizeof(*neg));
 	memcpy(neg->initiator, initiator, ETH_ALEN);
 	memcpy(neg->peer, peer, ETH_ALEN);
 	neg->state = PEER;
@@ -2006,10 +2005,10 @@ static void __wpa_send_eapol(struct wpa_authenticator *wpa_auth,
 		version = WPA_KEY_INFO_TYPE_HMAC_MD5_RC4;
 
 	wpa_printf(MSG_DEBUG, "WPA: Send EAPOL(secure=%d mic=%d ack=%d "
-		   "install=%d pairwise=%d ie_len=%d gtk_len=%d keyidx=%d "
+		   "install=%d pairwise=%d ie_len=%lu gtk_len=%lu keyidx=%d "
 		   "encr=%d)",
-		   secure, mic, ack, install, pairwise, ie_len, gtk_len,
-		   keyidx, encr);
+		   secure, mic, ack, install, pairwise, (unsigned long) ie_len,
+		   (unsigned long) gtk_len, keyidx, encr);
 
 	key_data_len = ie_len + gtk_len;
 
@@ -2025,11 +2024,10 @@ static void __wpa_send_eapol(struct wpa_authenticator *wpa_auth,
 
 	len += key_data_len;
 
-	hdr = malloc(len);
+	hdr = wpa_zalloc(len);
 	if (hdr == NULL)
 		return;
-	memset(hdr, 0, len);
-	hdr->version = EAPOL_VERSION;
+	hdr->version = wpa_auth->conf.eapol_version;
 	hdr->type = IEEE802_1X_TYPE_EAPOL_KEY;
 	hdr->length = htons(len  - sizeof(*hdr));
 	key = (struct wpa_eapol_key *) (hdr + 1);
@@ -2084,12 +2082,11 @@ static void __wpa_send_eapol(struct wpa_authenticator *wpa_auth,
 		memcpy(key + 1, ie, ie_len);
 		key->key_data_length = htons(ie_len);
 	} else if (encr && (gtk || ie)) {
-		buf = malloc(key_data_len);
+		buf = wpa_zalloc(key_data_len);
 		if (buf == NULL) {
 			free(hdr);
 			return;
 		}
-		memset(buf, 0, key_data_len);
 		pos = buf;
 		if (ie) {
 			memcpy(pos, ie, ie_len);
@@ -3142,3 +3139,5 @@ int wpa_auth_pmksa_add(struct wpa_state_machine *sm, const u8 *pmk,
 
 	return -1;
 }
+
+#endif /* CONFIG_NATIVE_WINDOWS */
